## Analysis script for:
# Heterochronic shifts mediate ecomorphological convergence in skull shape of microcephalic sea snakes
# In Integrative & Comparative Biology 2019


# Written November 2018 by Emma Sherratt

library(geomorph) # v3.0.7
library(abind) # v1.4-5
library(phytools) # v0.6-44
library(convevol) # v.1.3

# Read in coordinate data files
mydata <- readland.tps("Sea Snake Lmk coords 60.tps", specID = "ID")
# Negative landmark coordinates have been identified. Should they be considered as NAs? (y/n) n

# estimate missing lmks + GPAs
  mydata <- estimate.missing(mydata, method = "TPS")
  GPA <- gpagen(mydata) #GPA - for csize calculation only
  Csize <- GPA$Csize ; remove(GPA)
  # bilat symmetry GPA
  land.pairs <- read.csv("land pairs 60.csv", header=FALSE)
  GPA <- bilat.symmetry(mydata, ind = dimnames(mydata)[[3]], object.sym = TRUE, 
                        land.pairs = land.pairs, iter=1) 
  dimnames(GPA$symm.shape)[[3]] <- sub("ind", "", dimnames(GPA$symm.shape)[[3]]) #relabel specimens

# extract names
  classifiers <- data.frame(abrev = substr(dimnames(GPA$symm.shape)[[3]], 1,7), 
                            row.names = dimnames(GPA$symm.shape)[[3]], stringsAsFactors =F)
  classifiers$age <- "adult"
  classifiers$age[grep("_juv", dimnames(GPA$symm.shape)[[3]])] <- "juvenile"
  classifiers$age[which(rownames(classifiers)== "Hyd_sto_FMNH_202826")] <- "juvenile"

# get all the phylogeny abbreviations
  phy.abrevs <- read.csv("species classifier.csv", header=T, row.names = 1)
  classifiers <- data.frame(classifiers, phyloID1=rownames(phy.abrevs)[match(classifiers$abrev, phy.abrevs$species_abrev_2)])
  classifiers <- data.frame(classifiers, phyloID2=phy.abrevs$species_abrev[match(classifiers$abrev, phy.abrevs$species_abrev_2)])
  
# make as factor
  classifiers$abrev <- as.factor(classifiers$abrev)

# Reorder to match classifier
  Csize <- Csize[match(dimnames(GPA$symm.shape)[[3]], names(Csize))]

# Create adult only dataset  
  Ad.symm.shape <- GPA$symm.shape[,,which(classifiers$age == "adult")]
  Ad.classifiers <- classifiers[which(classifiers$age == "adult"),]
  Ad.Csize <- Csize[which(classifiers$age == "adult")]

### Plotting prep

# Create vector to colour spp in plot
  colours <- sample(rainbow(length(levels(classifiers$abrev))))
  names(colours) <- levels(classifiers$abrev)
  col <- colours[match(classifiers$abrev, names(colours))] 
  
# Create vector to show age groups in plot
  age.pch <- rep(21, length(classifiers$abrev)) # circles
  age.pch[which(classifiers$age == "juvenile")] <- 24 # triangles
  names(age.pch) <- classifiers$abrev
  
# Create vector to colour juvs & adults in plot
  JA.col <- rep("grey", length(classifiers$abrev))
  JA.col[which(classifiers$abrev == "Hyd_cur")] <- "black"
  JA.col[which(classifiers$abrev == "Hyd_sto")] <- "dark red"
  JA.col[which(classifiers$abrev == "Hyd_maj")] <- "red"
  JA.col[which(classifiers$abrev == "Hyd_ele")] <- "pink"
  JA.col[which(classifiers$abrev == "Hdr_dar")] <- "orange"
  JA.col[which(classifiers$abrev == "Hyd_don")] <- "yellow"
  JA.col[which(classifiers$abrev == "Hyd_oce")] <- "green"
  JA.col[which(classifiers$abrev == "Hyd_per")] <- "purple"
  JA.col[which(classifiers$abrev == "Hyd_fas")] <- "blue"
  JA.col[which(classifiers$abrev == "Hyd_mac")] <- "cyan"
  names(JA.col) <- classifiers$abrev
  
# geomorph dataframe for later
  gdf <- geomorph.data.frame(coords = GPA$symm.shape, Csize = Csize, 
                             species = classifiers$abrev)
  Ad.gdf <- geomorph.data.frame(coords = Ad.symm.shape, Csize = Ad.Csize, 
                                species = Ad.classifiers$abrev)
  
##### ADULT ONLY ANALYSES AND GRAPHS #####
# Multivariate regression of adults
Ad.allom <- procD.allometry(coords~Csize, logsz = TRUE, data = Ad.gdf)
summary(Ad.allom) #12.4%
# Type I (Sequential) Sums of Squares and Cross-products
# Randomized Residual Permutation Procedure Used
# 1000 Permutations
# ANOVA effect sizes and P-values based on empirical F distributions
# 
# 
#             Df      SS       MS     Rsq      F      Z Pr(>F)   
#   log(size)   1 0.11362 0.113621 0.12399 17.126 6.0454  0.001 **
#   Residuals 121 0.80276 0.006634 0.87601                        
#   Total     122 0.91638 

evo.onto.allom <- procD.allometry(coords~Csize, logsz = TRUE, data = gdf)
summary(evo.onto.allom)
# Type I (Sequential) Sums of Squares and Cross-products
# Randomized Residual Permutation Procedure Used
# 1000 Permutations
# ANOVA effect sizes and P-values based on empirical F distributions
# 
# 
#               Df      SS       MS     Rsq      F      Z Pr(>F)   
#   log(size)   1 0.13249 0.132492 0.12795 19.807 6.4261  0.001 **
#   Residuals 135 0.90304 0.006689 0.87205                        
#   Total     136 1.03553   

############################################################################################################
# Evolutionary Analyses
# read in tree
tree <- read.tree("sea snake MCC tree") # from https://doi.org/10.5061/dryad.48r5h.2
  plot(tree)

# average shape data
phylo.y <- (aggregate(two.d.array(Ad.symm.shape) ~ Ad.classifiers$phyloID1, FUN=mean))[,-1]
  rownames(phylo.y) <- levels(Ad.classifiers$phyloID1)
phylo.Y <- arrayspecs(phylo.y, ncol(phylo.y)/3, 3) ; remove(phylo.y)
phylo.csize <- (aggregate(Ad.Csize ~ Ad.classifiers$phyloID1, FUN=mean))[,-1]
  names(phylo.csize) <- levels(Ad.classifiers$phyloID1)
  
# New classifier table for just species data
phylo.class <- data.frame(abrev = unique(Ad.classifiers$abrev), phyloID1=unique(Ad.classifiers$phyloID1), phyloID2=unique(Ad.classifiers$phyloID2))
  row.names(phylo.class) <- unique(Ad.classifiers$phyloID1)
phylo.class <- phylo.class[dimnames(phylo.Y)[[3]],] # reorder
match(row.names(phylo.class), dimnames(phylo.Y)[[3]])  # check
  
# Prune to dataset & reorder to match tree tip.labels
  omit <- tree$tip.label[which(is.na(match(tree$tip.label,phylo.class$phyloID1)))]
  tree <- drop.tip(tree, omit) # prune tree to match dataset
  phylo.Y <- phylo.Y[,,tree$tip.label] # reorder data to match tree
  phylo.csize <- phylo.csize[tree$tip.label] # reorder
  phylo.class <- phylo.class[match(tree$tip.label,phylo.class$phyloID1),] # reorder

# Create vector to colour micros and nonmicros in plot (7 micro species out of 10)
  phy.micro.col <- rep("red", length(phylo.class$abrev))
  phy.micro.col[which(phylo.class$abrev == "Hyd_atr")] <- "blue"
  phy.micro.col[which(phylo.class$abrev == "Hyd_bro")] <- "blue"
  phy.micro.col[which(phylo.class$abrev == "Hyd_cog")] <- "blue"
  phy.micro.col[which(phylo.class$abrev == "Hyd_fas")] <- "blue"
  phy.micro.col[which(phylo.class$abrev == "Hyd_mac")] <- "blue"
  phy.micro.col[which(phylo.class$abrev == "Hyd_mel")] <- "blue"
  phy.micro.col[which(phylo.class$abrev == "Mic_gra")] <- "blue"
  names(phy.micro.col) <- phylo.class$abrev
  
  phylo.PCA <- plotTangentSpace(phylo.Y, label = dimnames(phylo.Y)[[3]], warpgrids = FALSE,
                                groups = phy.micro.col)

# Read in trait data, file from https://doi.org/10.5061/dryad.48r5h.2
  traitdata <- read.csv("trait data.csv", header=T, row.names = 1, stringsAsFactors = FALSE)
  traitdata <- traitdata[match(phylo.class$phyloID1,rownames(traitdata)),]

# Evolutionary allometry
  phy.gdf <- geomorph.data.frame(coords = phylo.Y, Csize = phylo.csize, 
                                 maxTL = traitdata$max.total.length..mm.,
                                 relgirth = traitdata$relative.girth, 
                                 prop.burr = traitdata$proportion.burrowing)

  # Test for evolutionary allometry
  phy.allom <- procD.pgls(coords ~ log(Csize), tree, data = phy.gdf)
  summary(phy.allom) # stats  16.4%
  # Type I (Sequential) Sums of Squares and Cross-products
  # Randomized Residual Permutation Procedure Used
  # 1000 Permutations
  # ANOVA effect sizes and P-values based on empirical F distributions
  # 
  # 
  #               Df       SS        MS     Rsq      F      Z Pr(>F)   
  #   log(Csize)  1 0.008756 0.0087556 0.16445 5.5107 3.8517  0.001 **
  #   Residuals  28 0.044488 0.0015888 0.83555                        
  #   Total      29 0.053243 
  plot(phy.allom, type = "regression", predictor = phy.gdf$Csize, reg.type = "RegScore") 

# Test for Convergent Evolution
  convtips <- c("Hydrophis_macdowelli", "Hydrophis_coggeri", "Hydrophis_atriceps", 
                "Hydrophis_brookii", "Hydrophis_fasciatus", "Hydrophis_melanocephalus",
                "Microcephalophis_gracilis")
  convrat(tree, phylo.PCA$pc.scores[,1:6], convtips)
  # C1          C2          C3          C4
  # 0.136176005 0.009906919 0.064503673 0.005773905 

  convratsig(tree, phylo.PCA$pc.scores[,1:6], convtips, nsim=1000) # takes a long time
  # $ObservedCs
  # C1          C2          C3          C4 
  # 0.147362787 0.010750530 0.069288960 0.006489906 
  # 
  # $Pvals
  # C1          C2          C3          C4 
  # 0.008991009 0.001998002 0.012987013 0.023976024 

  convnum(tree, phylo.PCA$pc.scores[,1:6], convtips, plot = TRUE, ellipse = NULL,
          plotellipse = NULL)

# Create vectors to colour spp in plot by diet
  rbPal <- colorRampPalette(c("white", "black"))
  diet.col <- rbPal(3)[as.numeric(cut(traitdata$proportion.burrowing,breaks = 3))]
  diet.col2 <- diet.col
  names(diet.col2) <- tree$tip.label
  diet.col2 <- diet.col2[match(classifiers$phyloID1, names(diet.col2))]
  names(diet.col2) <- classifiers$abrev
  
  diet.col3 <- diet.col2
  diet.col3[!is.na(match(classifiers$phyloID1, tree$tip.label[which(traitdata$prey.type == "gobies" & traitdata$proportion.burrowing > 0,7)]))] <- "light blue"
  diet.col3[!is.na(match(classifiers$phyloID1, tree$tip.label[which(traitdata$prey.type == "eels" & traitdata$proportion.burrowing > 0.7)]))] <- "yellow"
  
  diet.col4 <- diet.col3
  diet.col4[which(diet.col4 == '#FFFFFF')]  <- '#d9d9d9'
  
  phy.gdf <- geomorph.data.frame(phy.gdf, diet1 = traitdata$proportion.burrowing, diet2 = traitdata$prey.type)
  procD.pgls(coords ~ diet1, tree, data = phy.gdf)
  #             Df       SS        MS     Rsq     F      Z Pr(>F)   
  #   diet1      1 0.007135 0.0071352 0.13385 4.327 3.2394  0.001 **
  #   Residuals 28 0.046172 0.0016490 0.86615                       
  #   Total     29 0.053307  
  procD.pgls(log(Csize) ~ diet1, tree, data = phy.gdf)
  #            Df      SS      MS     Rsq      F      Z Pr(>F)   
  #   diet1      1 0.38653 0.38653 0.63945 49.658 2.3095  0.001 **
  #   Residuals 28 0.21795 0.00778 0.36055                        
  #   Total     29 0.60448  

# Figure 2
# Plot PCA Phylomorphospace PC1 vs PC2
x <- phylo.PCA$pc.scores[,1:2]
anc.states <- NULL
for (i in 1:ncol(x)) {
  x1 <- x[, i]
  tmp <- vector()
  for (j in 1:tree$Nnode + length(tree$tip.label)) {
    a <- multi2di(root(tree, node = j))
    tmp[j - length(tree$tip.label)] <- ace(x1, a, method = "pic")$ace[1]
  }
  anc.states <- cbind(anc.states, tmp)
}
colnames(anc.states) <- NULL ; row.names(anc.states) <- 1:length(tmp)
all.data <- rbind(x, anc.states)
# Plot
plot(all.data,type="n", xlab="PC1 (29.4%)", ylab= "PC2 (13.2%)", asp=TRUE)
for (i in 1:nrow(tree$edge)) {
  lines(all.data[(tree$edge[i, ]), 1], all.data[(tree$edge[i, ]), 2], type = "l", col = "grey", lwd = 1)}
points(all.data[((length(tree$tip.label)+1):nrow(all.data)),], pch=18,cex=1, col="grey")
points(all.data[which(traitdata$prey.type == "gobies" & traitdata$proportion.burrowing > 0,7),], pch=21,cex=2.4, bg="light blue")
points(all.data[which(traitdata$prey.type == "eels" & traitdata$proportion.burrowing > 0.7),], pch=21,cex=2.4, bg="yellow")
points(all.data[(1:length(tree$tip.label)),], pch=21,cex=1.4,bg=diet.col)
# # tip labels
for(i in 1:length(tree$tip.label)){
  x <- ifelse(all.data[i,1] > 7, 4, 2)
  text(all.data[i, 1], all.data[i, 2], tree$tip.label[i], cex= 1, pos=x, offset=0.5)}

# Figure 3A
# Plot All dataset multi regression with these diet colours on convex hulls
plot(log(Ad.allom$size),Ad.allom$Reg.proj, type = "n", ylab = "Regression Score", xlab="log-skull size")
plot(log(evo.onto.allom$size),evo.onto.allom$Reg.proj, type = "n", ylab = "Regression Score", xlab="log-skull size")
X <- cbind(log(Ad.allom$size),Ad.allom$Reg.proj)
Y <- Ad.classifiers$abrev
col <- diet.col2
col2 <- diet.col3
col3 <- diet.col4
for(i in levels(Y)){
  Z <- which(Y == i)
  if(length(Z) == 1) { points(X[Z,1], X[Z,2], pch=23, col = col[which(names(col) == i)], 
                              bg=adjustcolor(col2[which(names(col2) == i)], alpha.f = 0.3), cex=1)}
  else{ points(X[Z,c(1:2)], pch=21, bg=col[which(names(col) == i)], cex=1)
    EP <- rownames(X[Z,])[chull(X[Z, c(1,2)])] # find outer edge points
    if(is.na(diet.col4[Z])){
      polygon(X[EP,c(1,2)], col=adjustcolor(col2[which(names(col2) == i)], 
                                            alpha.f = 0.3), lty = 2, border='#d9d9d9') # Plot convex hull as polygon
    }
    polygon(X[EP,c(1,2)], col=adjustcolor(col2[which(names(col2) == i)], 
                                          alpha.f = 0.3), border=col3[which(names(col3) == i)]) # Plot convex hull as polygon
  }}
text(x=3.1, y=0.07, labels = "A", cex= 2)
legend(3.95,-0.01, legend = c("0 - 0.33","0.33 - 0.69","0.7 - 1"), 
       fill= rbPal(3), 
       title="proportion of burrowing prey",bty = "n")
legend(3.95,-0.04, legend = c("gobies", "burrowing eels"), 
       fill= c("light blue", "yellow"), 
       title="burrowing prey type",bty = "n")

# Figure 3B
plot(log(evo.onto.allom$size), evo.onto.allom$Reg.proj, pch= age.pch, bg=JA.col, cex=2, xlab="log-skull size", ylab="Regression Score")
